@@ -121,6 +121,18 @@ class SyntaxTransformer(ast.NodeTransformer):
         self.generic_visit(node)
         return node
 
+    def visit_FunctionDef(self, node):
+        return node
+
+    def visit_ClassDef(self, node):
+        return node
+
+    def visit_Import(self, node):
+        return node
+
+    def visit_ImportFrom(self, node):
+        return node
+
 
 class RenameTransformer(ast.NodeTransformer):
     """改名处理。改名前需要语法规范"""
@@ -308,6 +320,18 @@ class RenameTransformer(ast.NodeTransformer):
         self.generic_visit(node)
         return node
 
+    def visit_FunctionDef(self, node):
+        return node
+
+    def visit_ClassDef(self, node):
+        return node
+
+    def visit_Import(self, node):
+        return node
+
+    def visit_ImportFrom(self, node):
+        return node
+
 
 def source_replace(source: str) -> str:
     # 三元表达式转换成 错误版if( )else，一定得在Transformer中修正
@@ -347,7 +371,7 @@ def assigns_to_list(assigns):
 
 def raw_to_code(raw):
     """导入语句转字符列表"""
-    return '\n'.join([ast.unparse(a) for a in raw])
+    return '\n'.join([ast_comments.unparse(a) for a in raw])
 
 
 def sources_to_asts(*sources, convert_xor: bool):
@@ -381,15 +405,16 @@ def sources_to_asts(*sources, convert_xor: bool):
         if isinstance(node, ast.Assign):
             assigns.append(node)
             continue
-        if isinstance(node, ast_comments.Comment):
-            # 添加注释
-            if node.inline and isinstance(assigns[-1], ast.Assign):
-                assigns.append(node)
-                continue
         # TODO 是否要把其它语句也加入？是否有安全问题？
-        if isinstance(node, (ast.Import, ast.ImportFrom)):
+        if isinstance(node, (ast.Import, ast.ImportFrom, ast.FunctionDef, ast.ClassDef)):
             raw.append(node)
             continue
+        if isinstance(node, ast_comments.Comment):
+            # 添加注释
+            if node.inline and isinstance(tree.body[i - 1], ast.Assign):
+                assigns.append(node)
+                continue
+
     return raw_to_code(raw), assigns_to_list(assigns), t.funcs_new, t.args_new, t.targets_new
 
 
