@@ -49,11 +49,10 @@ Expr.diff = _diff
 # ===============================
 
 def simplify2(expr):
-    # return simplify(expr)
     try:
         expr = simplify(expr)
-    except AttributeError as e:
-        print(f'{expr} ,表达式无法简化, {e}')
+    except (AttributeError, TypeError) as e:
+        logger.warning('{}, 表达式无法简化, {}', expr, repr(e))
     return expr
 
 
@@ -202,7 +201,9 @@ class ExprTool:
             replace: bool = True, regroup: bool = False, format: bool = True,
             date='date', asset='asset',
             extra_codes: Sequence[object] = (),
+            over_null: Literal['order_by', 'partition_by', None] = 'partition_by',
             table_name: str = 'self',
+            filter_last: bool = False,
             **kwargs):
         """功能集成版，将几个功能写到一起方便使用
 
@@ -226,6 +227,8 @@ class ExprTool:
             资产字段名
         extra_codes: Sequence[object]
             需要复制到模板中的额外代码
+        table_name
+        filter_last
 
         Returns
         -------
@@ -264,7 +267,9 @@ class ExprTool:
         codes = codegen(exprs_ldl, exprs_src, syms_dst,
                         filename=template_file, date=date, asset=asset,
                         extra_codes=extra_codes,
+                        over_null=over_null,
                         table_name=table_name,
+                        filter_last=filter_last,
                         **kwargs)
 
         logger.info(f'{style} code is generated')
@@ -284,7 +289,9 @@ class ExprTool:
                   style: Literal['pandas', 'polars', 'sql'] = 'polars',
                   template_file: Optional[str] = None,
                   date: str = 'date', asset: str = 'asset',
+                  over_null: Literal['order_by', 'partition_by', None] = 'partition_by',
                   table_name: str = 'self',
+                  filter_last: bool = False,
                   **kwargs) -> str:
         """通过字符串生成代码， 加了缓存，多次调用不重复生成"""
         raw, exprs_list = sources_to_exprs(self.globals_, source, *more_sources, convert_xor=convert_xor)
@@ -298,12 +305,13 @@ class ExprTool:
                                           # 传入多个列的方法
                                           extra_codes,
                                           ),
+                             over_null=over_null,
                              table_name=table_name,
+                             filter_last=filter_last,
                              **kwargs)
 
         # 移回到cache，防止多次调用多次保存
-        if isinstance(output_file, TextIOBase):
-            # 输出到控制台
+        if hasattr(output_file, "write"):
             output_file.write(code)
         elif output_file is not None:
             output_file = pathlib.Path(output_file)
@@ -351,7 +359,7 @@ def _get_code_from_file(file: str):
 _TOOL_ = ExprTool()
 
 
-def codegen_exec(df: Optional[DataFrame],
+def codegen_exec(df: Union[DataFrame, None],
                  *codes,
                  over_null: Literal['partition_by', 'order_by', None],
                  extra_codes: str = r'CS_SW_L1 = r"^sw_l1_\d+$"',
@@ -362,7 +370,8 @@ def codegen_exec(df: Optional[DataFrame],
                  template_file: Optional[str] = None,
                  date: str = 'date', asset: str = 'asset',
                  table_name: str = 'self',
-                 **kwargs) -> Union[DataFrame, str, None]:
+                 filter_last: bool = False,
+                 **kwargs) -> Union[DataFrame, str]:
     """快速转换源代码并执行
 
     Parameters
@@ -395,12 +404,15 @@ def codegen_exec(df: Optional[DataFrame],
     asset: str
         资产字段
     over_null: str
-        时序中遇到null时的处理方式
+        时序中遇到null时的处理方式。只在style参数为'polars', 'sql'时有效
         - partition_by: 空值划分到不同分区
         - order_by: 空值排同一分区的前排
         - None: 不做处理
     table_name:str
-        表名。style=sql时有效
+        表名。只在style参数为sql时有效
+    filter_last:bool
+        在实盘时，只需要最后一天日期的数据，可以在最后一个`ts`之后过滤数据。目前只在style参数为'polars', 'pandas'时有效
+
 
     Returns
     -------
@@ -453,6 +465,7 @@ def codegen_exec(df: Optional[DataFrame],
         date=date, asset=asset,
         over_null=over_null,
         table_name=table_name,
+        filter_last=filter_last,
         **kwargs
     )
 
